"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = void 0;
exports.rcOptionsTypes = rcOptionsTypes;
exports.cliOptionsTypes = cliOptionsTypes;
exports.help = help;
exports.handler = handler;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const cli_utils_1 = require("@pnpm/cli-utils");
const cli_meta_1 = require("@pnpm/cli-meta");
const client_1 = require("@pnpm/client");
const pick_registry_for_package_1 = require("@pnpm/pick-registry-for-package");
const config_1 = require("@pnpm/config");
const error_1 = require("@pnpm/error");
const logger_1 = require("@pnpm/logger");
const plugin_commands_installation_1 = require("@pnpm/plugin-commands-installation");
const read_project_manifest_1 = require("@pnpm/read-project-manifest");
const tools_path_1 = require("@pnpm/tools.path");
const link_bins_1 = require("@pnpm/link-bins");
const pick_1 = __importDefault(require("ramda/src/pick"));
const render_help_1 = __importDefault(require("render-help"));
function rcOptionsTypes() {
    return (0, pick_1.default)([], config_1.types);
}
function cliOptionsTypes() {
    return {
        ...rcOptionsTypes(),
    };
}
exports.commandNames = ['self-update'];
function help() {
    return (0, render_help_1.default)({
        description: 'Updates pnpm to the latest version (or the one specified)',
        descriptionLists: [],
        url: (0, cli_utils_1.docsUrl)('self-update'),
        usages: [
            'pnpm self-update',
            'pnpm self-update 9',
            'pnpm self-update next-10',
            'pnpm self-update 9.10.0',
        ],
    });
}
async function handler(opts, params) {
    if ((0, cli_meta_1.isExecutedByCorepack)()) {
        throw new error_1.PnpmError('CANT_SELF_UPDATE_IN_COREPACK', 'You should update pnpm with corepack');
    }
    const { resolve } = (0, client_1.createResolver)({ ...opts, authConfig: opts.rawConfig });
    const pkgName = 'pnpm';
    const pref = params[0] ?? 'latest';
    const resolution = await resolve({ alias: pkgName, pref }, {
        lockfileDir: opts.lockfileDir ?? opts.dir,
        preferredVersions: {},
        projectDir: opts.dir,
        registry: (0, pick_registry_for_package_1.pickRegistryForPackage)(opts.registries, pkgName, pref),
    });
    if (!resolution?.manifest) {
        throw new error_1.PnpmError('CANNOT_RESOLVE_PNPM', `Cannot find "${pref}" version of pnpm`);
    }
    if (resolution.manifest.version === cli_meta_1.packageManager.version) {
        return `The currently active ${cli_meta_1.packageManager.name} v${cli_meta_1.packageManager.version} is already "${pref}" and doesn't need an update`;
    }
    if (opts.wantedPackageManager?.name === cli_meta_1.packageManager.name && opts.managePackageManagerVersions) {
        const { manifest, writeProjectManifest } = await (0, read_project_manifest_1.readProjectManifest)(opts.rootProjectManifestDir);
        manifest.packageManager = `pnpm@${resolution.manifest.version}`;
        await writeProjectManifest(manifest);
        return `The current project has been updated to use pnpm v${resolution.manifest.version}`;
    }
    const currentPkgName = (0, cli_meta_1.getCurrentPackageName)();
    const dir = (0, tools_path_1.getToolDirPath)({
        pnpmHomeDir: opts.pnpmHomeDir,
        tool: {
            name: currentPkgName,
            version: resolution.manifest.version,
        },
    });
    if (fs_1.default.existsSync(dir)) {
        await (0, link_bins_1.linkBins)(path_1.default.join(dir, opts.modulesDir ?? 'node_modules'), opts.pnpmHomeDir, {
            warn: logger_1.globalWarn,
        });
        return `The ${pref} version, v${resolution.manifest.version}, is already present on the system. It was activated by linking it from ${dir}.`;
    }
    fs_1.default.mkdirSync(dir, { recursive: true });
    fs_1.default.writeFileSync(path_1.default.join(dir, 'package.json'), '{}');
    await plugin_commands_installation_1.add.handler({
        ...opts,
        dir,
        lockfileDir: dir,
        bin: opts.pnpmHomeDir,
    }, [`${currentPkgName}@${resolution.manifest.version}`]);
    return undefined;
}
//# sourceMappingURL=selfUpdate.js.map